# https://dashscope.console.aliyun.com/model
import json
from typing import Any
from remote_engine import BaseRemoteEngine
from plugins_conf import PluginsConfig

from logging import getLogger
logger = getLogger(__name__)

PluginsConfig().add_core_path()
from core.base_chat_ai_engine import BaseChatAIEngine

class ChatAiTongYiQianWen(BaseChatAIEngine, BaseRemoteEngine):
    API_URL = "https://dashscope.aliyuncs.com/api/v1/services/aigc/text-generation/generation"
    def __init__(self, api_key : str,
                 model_name : str = "qwen-1.8b-chat",
                 is_stream : bool = False,
                 *args, **kwargs) -> None:
        self._modual_name = model_name
        BaseChatAIEngine.__init__(self, is_stream=is_stream)
        BaseRemoteEngine.__init__(self, api_key=api_key,
                         base_url=self.API_URL)

    def _transform_raw(self, response):
        return response["output"]["choices"][0]["message"]["content"]

    def _transform_iterator(self, response):
        for chunk in response:
            input_string = chunk.decode('utf-8')
            json_string = input_string[input_string.find("data:") + 5:].strip()
            yield self._transform_raw(json.loads(json_string))

    def transform(self, data):
        return self._transform_iterator(data) if self._is_stream else\
            self._transform_raw(data)
    
    def execute(self, data: Any) -> Any:
        if self._is_stream:
            self.header['X-DashScope-SSE'] = 'enable'
        response = self.post_prompt({
                'model': self._modual_name,
                "input": {
                    "messages": data
                },
                "parameters": {
                    "result_format": "message",
                    "incremental_output": self._is_stream
                }
            },
            is_stream=self._is_stream)
        if self._is_stream:
            ret = response.iter_content(chunk_size=None)
        else:
            ret = response.json()
        return ret