from pathlib import Path
import importlib
import sys
from typing import Any, Callable, Dict, Optional, List

import yaml
CURRENT_DIR_PATH = Path(__file__).resolve().parent

class PluginsConfig:
    def __init__(self,
                 base_path : Path | str = CURRENT_DIR_PATH,
                 conf_name : str = ".plugins.yml"):
        base_path = Path(base_path)
        self.base_path = base_path.resolve()
        self.conf_path = base_path / conf_name
        if not self.conf_path.exists():
            raise FileExistsError(f"{self.conf_path} not exists")
        if not self.conf_path.is_file():
            raise TypeError(f"{self.conf_path} is not a file")
        self.config:Dict = yaml.safe_load(self.conf_path.read_text("utf-8"))
        global_conf = self.config.get('global')
        self.core_path = self.base_path / global_conf.get('core_path')
        self.plugin_path = self.base_path / global_conf.get('plugin_path')
        sys.path.append(str(self.base_path))

    def add_core_path(self):
        sys.path.append(str(self.core_path.resolve()))

    def get_plugin_full_name(self, engine_name):
        find_name = 'plugin'
        plugin_name = self.config[engine_name][find_name]
        if not plugin_name:
            raise ValueError(f"{engine_name}.{find_name} is not found in {self.conf_path}")
        return plugin_name

    def get_engine_modual(self, engine_name):
        plugin_name = self.get_plugin_full_name(engine_name)
        sys.path.append(str(self.plugin_path.resolve()))
        modual = importlib.import_module(plugin_name)
        return modual
    
    def get_engine_config(self, engine_name):
        plugin_name = self.get_plugin_full_name(engine_name)
        return self.config[engine_name][plugin_name]
    
    def get_engine_class(self, engine_name):
        engine_modual = self.get_engine_modual(engine_name)
        engine_config = self.get_engine_config(engine_name)
        engine = getattr(engine_modual, engine_config['class_name'])
        return engine(**engine_config)